@@ -1,10 +1,12 @@
 """Tests for the signed cookie module for Starlette."""
+# pylint: disable=R0801
 
 import typing
 from abc import abstractmethod
 from unittest import mock
 
 import pytest
+from blake2signer.errors import InvalidSignatureError
 from starlette.applications import Starlette
 from starlette.middleware import Middleware
 from starlette.requests import Request
@@ -32,7 +34,7 @@ class SignedCookieMiddlewareTestsForStarletteBase(typing.Generic[TMiddleware, TD
     def middleware_class(self) -> typing.Type[TMiddleware]:
         """Get this middleware class."""
         return typing.get_args(
-            type(self).__orig_bases__[0],  # type: ignore
+            type(self).__orig_bases__[0],  # type: ignore  # pylint: disable=E1101
         )[0]
 
     @abstractmethod
@@ -56,17 +58,17 @@ class SignedCookieMiddlewareTestsForStarletteBase(typing.Generic[TMiddleware, TD
         else:
             state_attr = state_attribute_name
 
-        def root(request: 'Request') -> JSONResponse:
+        def root(request: Request) -> JSONResponse:  # pylint: disable=W0613
             """Main app endpoint."""
             return JSONResponse({
                 'hello': 'world',
             })
 
-        def cookie_endpoint(request: 'Request') -> JSONResponse:
+        def cookie_endpoint(request: Request) -> JSONResponse:
             """Endpoint that writes a cookie."""
             cookie_data = getattr(request.state, state_attr)
-            modified_data = self.modify_cookie_value(cookie_data)
-            setattr(request.state, state_attr, modified_data)
+            modified_data = self.modify_cookie_value(cookie_data.data)
+            cookie_data.data = modified_data
 
             return JSONResponse()
 
@@ -77,7 +79,7 @@ class SignedCookieMiddlewareTestsForStarletteBase(typing.Generic[TMiddleware, TD
         if routes:
             all_routes.extend(routes)
 
-        middleware = [
+        middleware = (
             Middleware(
                 self.middleware_class,
                 secret=self.secret if secret is None else secret,
@@ -87,7 +89,7 @@ class SignedCookieMiddlewareTestsForStarletteBase(typing.Generic[TMiddleware, TD
                 cookie_properties=cookie_properties,
                 signer_kwargs=signer_kwargs,
             ),
-        ]
+        )
 
         app = Starlette(debug=True, routes=all_routes, middleware=middleware)
 
@@ -120,9 +122,10 @@ class SignedCookieMiddlewareTestsForStarletteBase(typing.Generic[TMiddleware, TD
     def test_state_is_used(self) -> None:
         """Test that `request.state` is used."""
 
-        def state_endpoint(request: 'Request') -> JSONResponse:
+        def state_endpoint(request: Request) -> JSONResponse:
             """Endpoint that asserts the state value."""
-            assert 'existing' == request.state.msgs
+            assert 'existing' == request.state.msgs.data
+            assert request.state.msgs.exc is None
 
             return JSONResponse()
 
@@ -148,10 +151,7 @@ class SignedCookieMiddlewareTestsForStarletteBase(typing.Generic[TMiddleware, TD
         """Test that the middleware does not interfere with endpoints that won't use it."""
         client = self.create_test_client()
 
-        with mock.patch.object(
-                self.middleware_class,
-                'write_cookie',
-        ) as mock_write_cookie:
+        with mock.patch.object(self.middleware_class, 'write_cookie') as mock_write_cookie:
             response = client.get('/')
 
         assert 200 == response.status_code
@@ -181,13 +181,19 @@ class SignedCookieMiddlewareTestsForStarletteBase(typing.Generic[TMiddleware, TD
                 ValueError,
                 match='The `secret` should not be included in the signer kwargs',
         ):
-            client.get('/')
+            client.get(
+                '/',
+                cookies={self.cookie_name: 'some value'},
+            )
 
         with pytest.raises(
                 ValueError,
                 match='The `secret` should not be included in the signer kwargs',
         ):
-            client.get('/cookie')
+            client.get(
+                '/cookie',
+                cookies={self.cookie_name: 'some value'},
+            )
 
     def test_existing_cookie_is_read_wrong_signature(self) -> None:
         """Test that existing cookie is read with wrong signature."""
@@ -207,8 +213,8 @@ class SignedCookieMiddlewareTestsForStarletteBase(typing.Generic[TMiddleware, TD
         assert 200 == response.status_code
         assert response.json() is None
         mock_should_write_cookie.assert_called_once_with(
-            unsigned_data=None,  # data in cookie is ignored
-            state_data=self.modify_cookie_value(None),
+            prev_data=None,  # data in cookie is ignored
+            new_data=self.modify_cookie_value(None),
         )
 
     def test_cookie_is_set_with_properties(self) -> None:
@@ -242,6 +248,172 @@ class SignedCookieMiddlewareTestsForStarletteBase(typing.Generic[TMiddleware, TD
             samesite='lax',
         )
 
+    def test_no_cookie_no_sig_check(self) -> None:
+        """Test that when there's no cookie, no signature is checked."""
+        client = self.create_test_client()
+
+        patch_target = f'asgi_signing_middleware.cookie.{self.middleware_class.__name__}.unsign'
+        with mock.patch(patch_target) as mock_unsign:
+            response = client.get('/')
+
+        assert response.status_code == 200
+        mock_unsign.assert_not_called()
+
+    def test_no_cookie_no_data(self) -> None:
+        """Test that when there's no cookie, data is null."""
+
+        def state_endpoint(request: Request) -> JSONResponse:
+            """Endpoint that asserts the state value."""
+            cookie_data = getattr(request.state, self.state_attribute_name)
+            assert cookie_data.data is None
+            assert cookie_data.exc is None
+
+            return JSONResponse()
+
+        client = self.create_test_client(
+            routes=[
+                Route('/state', state_endpoint),
+            ],
+        )
+
+        patch_target = f'asgi_signing_middleware.cookie.{self.middleware_class.__name__}.unsign'
+        with mock.patch(patch_target) as mock_unsign:
+            response = client.get('/state')
+
+        assert response.status_code == 200
+        mock_unsign.assert_not_called()
+
+    def test_cookie_read_with_sig_check(self) -> None:
+        """Test that when there's no cookie, no signature is checked."""
+        client = self.create_test_client()
+
+        patch_target = f'asgi_signing_middleware.cookie.{self.middleware_class.__name__}.unsign'
+        with mock.patch(patch_target, side_effect=InvalidSignatureError) as mock_unsign:
+            response = client.get(
+                '/',
+                cookies={self.cookie_name: 'some data'},
+            )
+
+        assert response.status_code == 200
+        mock_unsign.assert_called_once_with('some data')
+
+    def test_state_signer_exception(self) -> None:
+        """Test that we can read the signer exception from any handler."""
+
+        def state_endpoint(request: Request) -> JSONResponse:
+            """Endpoint that asserts the state value."""
+            cookie_data = getattr(request.state, self.state_attribute_name)
+            assert cookie_data.data is None
+            assert isinstance(cookie_data.exc, InvalidSignatureError)
+
+            return JSONResponse()
+
+        client = self.create_test_client(
+            routes=[
+                Route('/state', state_endpoint),
+            ],
+        )
+
+        patch_target = f'asgi_signing_middleware.cookie.{self.middleware_class.__name__}.unsign'
+        with mock.patch(patch_target, side_effect=InvalidSignatureError) as mock_unsign:
+            response = client.get(
+                '/state',
+                cookies={self.cookie_name: 'some data'},
+            )
+
+        assert response.status_code == 200
+        mock_unsign.assert_called_once_with('some data')
+
+    def test_state_expected_cookie_value(self) -> None:
+        """Test that we got the expected cookie value in the handler."""
+
+        def state_endpoint(request: Request) -> JSONResponse:
+            """Endpoint that asserts the state value."""
+            cookie_data = getattr(request.state, self.state_attribute_name)
+            assert cookie_data.data == 'some data'
+            assert cookie_data.exc is None
+
+            return JSONResponse()
+
+        client = self.create_test_client(
+            routes=[
+                Route('/state', state_endpoint),
+            ],
+        )
+
+        patch_target = f'asgi_signing_middleware.cookie.{self.middleware_class.__name__}.unsign'
+        with mock.patch(patch_target, return_value='some data') as mock_unsign:
+            response = client.get(
+                '/state',
+                cookies={self.cookie_name: 'some data'},
+            )
+
+        assert response.status_code == 200
+        mock_unsign.assert_called_once_with('some data')
+
+    def test_reset_cookie_data(self) -> None:
+        """Test that we can reset cookie data (setting it as blank)."""
+
+        def state_endpoint(request: Request) -> JSONResponse:
+            """Endpoint that asserts the state value."""
+            cookie_data = getattr(request.state, self.state_attribute_name)
+            assert cookie_data.data == 'some data'
+            assert cookie_data.exc is None
+
+            cookie_data.data = ''
+
+            return JSONResponse()
+
+        client = self.create_test_client(
+            routes=[
+                Route('/state', state_endpoint),
+            ],
+        )
+
+        patch_target = f'asgi_signing_middleware.cookie.{self.middleware_class.__name__}.unsign'
+        with mock.patch.object(self.middleware_class, 'write_cookie') as mock_write_cookie:
+            with mock.patch(patch_target, return_value='some data') as mock_unsign:
+                response = client.get(
+                    '/state',
+                    cookies={self.cookie_name: 'some data'},
+                )
+
+        assert response.status_code == 200
+        mock_unsign.assert_called_once_with('some data')
+        mock_write_cookie.assert_called_once()
+        assert mock_write_cookie.call_args[0][0] == ''
+
+    def test_do_not_reset_cookie_data(self) -> None:
+        """Test that we do not reset cookie data when setting it to null."""
+
+        def state_endpoint(request: Request) -> JSONResponse:
+            """Endpoint that asserts the state value."""
+            cookie_data = getattr(request.state, self.state_attribute_name)
+            assert cookie_data.data == 'some data'
+            assert cookie_data.exc is None
+
+            cookie_data.data = None
+
+            return JSONResponse()
+
+        client = self.create_test_client(
+            routes=[
+                Route('/state', state_endpoint),
+            ],
+        )
+
+        patch_target = f'asgi_signing_middleware.cookie.{self.middleware_class.__name__}.unsign'
+        with mock.patch.object(self.middleware_class, 'write_cookie') as mock_write_cookie:
+            with mock.patch(patch_target, return_value='some data') as mock_unsign:
+                response = client.get(
+                    '/state',
+                    cookies={self.cookie_name: 'some data'},
+                )
+
+        assert response.status_code == 200
+        mock_unsign.assert_called_once_with('some data')
+        mock_write_cookie.assert_not_called()
+
 
 class TestSimpleSignedCookieMiddlewareForStarlette(
         SignedCookieMiddlewareTestsForStarletteBase[SimpleSignedCookieMiddleware, str],
@@ -342,16 +514,19 @@ class TestSerializedSignedCookieMiddlewareForStarlette(
             return {
                 'extra': 'data',
             }
-        elif isinstance(data, dict):
+
+        if isinstance(data, dict):
             return {
                 **data,
                 **{
                     'extra': 'data',
                 },
             }
-        elif isinstance(data, list):
+
+        if isinstance(data, list):
             return [*data, *['extra']]
-        elif isinstance(data, (float, int)):
+
+        if isinstance(data, (float, int)):
             return data + 1
 
         return data + 'changed'
