@@ -2,6 +2,7 @@
 
 Note: this tests won't run under PyPy, due to orjson not being able to install under PyPy.
 """
+# pylint: disable=R0801
 
 import platform
 import typing
@@ -9,6 +10,7 @@ from abc import abstractmethod
 from unittest import mock
 
 import pytest
+from blake2signer.errors import InvalidSignatureError
 from starlette.middleware import Middleware
 
 from ..cookie import CookieProperties
@@ -40,7 +42,7 @@ class SignedCookieMiddlewareTestsForStarliteBase(typing.Generic[TMiddleware, TDa
     def middleware_class(self) -> typing.Type[TMiddleware]:
         """Get this middleware class."""
         return typing.get_args(
-            type(self).__orig_bases__[0],  # type: ignore
+            type(self).__orig_bases__[0],  # type: ignore  # pylint: disable=E1101
         )[0]
 
     @abstractmethod
@@ -65,18 +67,18 @@ class SignedCookieMiddlewareTestsForStarliteBase(typing.Generic[TMiddleware, TDa
             state_attr = state_attribute_name
 
         @get('/')
-        def root(request: 'Request') -> typing.Dict[str, str]:
+        def root(request: Request) -> typing.Dict[str, str]:  # pylint: disable=W0613
             """Main app endpoint."""
             return {
                 'hello': 'world',
             }
 
         @get('/cookie')
-        def cookie_endpoint(request: 'Request') -> None:
+        def cookie_endpoint(request: Request) -> None:
             """Endpoint that writes a cookie."""
             cookie_data = getattr(request.state, state_attr)
-            modified_data = self.modify_cookie_value(cookie_data)
-            setattr(request.state, state_attr, modified_data)
+            modified_data = self.modify_cookie_value(cookie_data.data)
+            cookie_data.data = modified_data
 
         route_handlers: typing.List[ControllerRouterHandler] = [
             root,
@@ -133,9 +135,10 @@ class SignedCookieMiddlewareTestsForStarliteBase(typing.Generic[TMiddleware, TDa
         """Test that `request.state` is used."""
 
         @get('/state')
-        def state_endpoint(request: 'Request') -> None:
+        def state_endpoint(request: Request) -> None:
             """Endpoint that asserts the state value."""
-            assert 'existing' == request.state.msgs
+            assert 'existing' == request.state.msgs.data
+            assert request.state.msgs.exc is None
 
         client = self.create_test_client(
             state_attribute_name='msgs',
@@ -188,17 +191,21 @@ class SignedCookieMiddlewareTestsForStarliteBase(typing.Generic[TMiddleware, TDa
             },
         )
 
-        with pytest.raises(
-                ValueError,
-                match='The `secret` should not be included in the signer kwargs',
-        ):
-            client.get('/')
+        response = client.get(
+            '/',
+            cookies={self.cookie_name: 'some value'},
+        )
+        assert response.status_code == 500
+        assert response.text.startswith('Traceback (most recent call last)')
+        assert response.text.endswith('The `secret` should not be included in the signer kwargs\n')
 
-        with pytest.raises(
-                ValueError,
-                match='The `secret` should not be included in the signer kwargs',
-        ):
-            client.get('/cookie')
+        response = client.get(
+            '/cookie',
+            cookies={self.cookie_name: 'some value'},
+        )
+        assert response.status_code == 500
+        assert response.text.startswith('Traceback (most recent call last)')
+        assert response.text.endswith('The `secret` should not be included in the signer kwargs\n')
 
     def test_existing_cookie_is_read_wrong_signature(self) -> None:
         """Test that existing cookie is read with wrong signature."""
@@ -218,8 +225,8 @@ class SignedCookieMiddlewareTestsForStarliteBase(typing.Generic[TMiddleware, TDa
         assert 200 == response.status_code
         assert response.json() is None
         mock_should_write_cookie.assert_called_once_with(
-            unsigned_data=None,  # data in cookie is ignored
-            state_data=self.modify_cookie_value(None),
+            prev_data=None,  # data in cookie is ignored
+            new_data=self.modify_cookie_value(None),
         )
 
     def test_cookie_is_set_with_properties(self) -> None:
@@ -253,6 +260,167 @@ class SignedCookieMiddlewareTestsForStarliteBase(typing.Generic[TMiddleware, TDa
             samesite='lax',
         )
 
+    def test_no_cookie_no_sig_check(self) -> None:
+        """Test that when there's no cookie, no signature is checked."""
+        client = self.create_test_client()
+
+        patch_target = f'asgi_signing_middleware.cookie.{self.middleware_class.__name__}.unsign'
+        with mock.patch(patch_target) as mock_unsign:
+            response = client.get('/')
+
+        assert response.status_code == 200
+        mock_unsign.assert_not_called()
+
+    def test_no_cookie_no_data(self) -> None:
+        """Test that when there's no cookie, data is null."""
+
+        @get('/state')
+        def state_endpoint(request: Request) -> None:
+            """Endpoint that asserts the state value."""
+            cookie_data = getattr(request.state, self.state_attribute_name)
+            assert cookie_data.data is None
+            assert cookie_data.exc is None
+
+        client = self.create_test_client(
+            handlers=[
+                state_endpoint,
+            ],
+        )
+
+        patch_target = f'asgi_signing_middleware.cookie.{self.middleware_class.__name__}.unsign'
+        with mock.patch(patch_target) as mock_unsign:
+            response = client.get('/state')
+
+        assert response.status_code == 200
+        mock_unsign.assert_not_called()
+
+    def test_cookie_read_with_sig_check(self) -> None:
+        """Test that when there's no cookie, no signature is checked."""
+        client = self.create_test_client()
+
+        patch_target = f'asgi_signing_middleware.cookie.{self.middleware_class.__name__}.unsign'
+        with mock.patch(patch_target, side_effect=InvalidSignatureError) as mock_unsign:
+            response = client.get(
+                '/',
+                cookies={self.cookie_name: 'some data'},
+            )
+
+        assert response.status_code == 200
+        mock_unsign.assert_called_once_with('some data')
+
+    def test_state_signer_exception(self) -> None:
+        """Test that we can read the signer exception from any handler."""
+
+        @get('/state')
+        def state_endpoint(request: Request) -> None:
+            """Endpoint that asserts the state value."""
+            cookie_data = getattr(request.state, self.state_attribute_name)
+            assert cookie_data.data is None
+            assert isinstance(cookie_data.exc, InvalidSignatureError)
+
+        client = self.create_test_client(
+            handlers=[
+                state_endpoint,
+            ],
+        )
+
+        patch_target = f'asgi_signing_middleware.cookie.{self.middleware_class.__name__}.unsign'
+        with mock.patch(patch_target, side_effect=InvalidSignatureError) as mock_unsign:
+            response = client.get(
+                '/state',
+                cookies={self.cookie_name: 'some data'},
+            )
+
+        assert response.status_code == 200
+        mock_unsign.assert_called_once_with('some data')
+
+    def test_state_expected_cookie_value(self) -> None:
+        """Test that we got the expected cookie value in the handler."""
+
+        @get('/state')
+        def state_endpoint(request: Request) -> None:
+            """Endpoint that asserts the state value."""
+            cookie_data = getattr(request.state, self.state_attribute_name)
+            assert cookie_data.data == 'some data'
+            assert cookie_data.exc is None
+
+        client = self.create_test_client(
+            handlers=[
+                state_endpoint,
+            ],
+        )
+
+        patch_target = f'asgi_signing_middleware.cookie.{self.middleware_class.__name__}.unsign'
+        with mock.patch(patch_target, return_value='some data') as mock_unsign:
+            response = client.get(
+                '/state',
+                cookies={self.cookie_name: 'some data'},
+            )
+
+        assert response.status_code == 200
+        mock_unsign.assert_called_once_with('some data')
+
+    def test_reset_cookie_data(self) -> None:
+        """Test that we can reset cookie data (setting it as blank)."""
+
+        @get('/state')
+        def state_endpoint(request: Request) -> None:
+            """Endpoint that asserts the state value."""
+            cookie_data = getattr(request.state, self.state_attribute_name)
+            assert cookie_data.data == 'some data'
+            assert cookie_data.exc is None
+
+            cookie_data.data = ''
+
+        client = self.create_test_client(
+            handlers=[
+                state_endpoint,
+            ],
+        )
+
+        patch_target = f'asgi_signing_middleware.cookie.{self.middleware_class.__name__}.unsign'
+        with mock.patch.object(self.middleware_class, 'write_cookie') as mock_write_cookie:
+            with mock.patch(patch_target, return_value='some data') as mock_unsign:
+                response = client.get(
+                    '/state',
+                    cookies={self.cookie_name: 'some data'},
+                )
+
+        assert response.status_code == 200
+        mock_unsign.assert_called_once_with('some data')
+        mock_write_cookie.assert_called_once()
+        assert mock_write_cookie.call_args[0][0] == ''
+
+    def test_do_not_reset_cookie_data(self) -> None:
+        """Test that we do not reset cookie data when setting it to null."""
+
+        @get('/state')
+        def state_endpoint(request: Request) -> None:
+            """Endpoint that asserts the state value."""
+            cookie_data = getattr(request.state, self.state_attribute_name)
+            assert cookie_data.data == 'some data'
+            assert cookie_data.exc is None
+
+            cookie_data.data = None
+
+        client = self.create_test_client(
+            handlers=[
+                state_endpoint,
+            ],
+        )
+
+        patch_target = f'asgi_signing_middleware.cookie.{self.middleware_class.__name__}.unsign'
+        with mock.patch.object(self.middleware_class, 'write_cookie') as mock_write_cookie:
+            with mock.patch(patch_target, return_value='some data') as mock_unsign:
+                response = client.get(
+                    '/state',
+                    cookies={self.cookie_name: 'some data'},
+                )
+
+        assert response.status_code == 200
+        mock_unsign.assert_called_once_with('some data')
+        mock_write_cookie.assert_not_called()
+
 
 class TestSimpleSignedCookieMiddlewareForStarlite(
         SignedCookieMiddlewareTestsForStarliteBase[SimpleSignedCookieMiddleware, str],
@@ -353,16 +521,19 @@ class TestSerializedSignedCookieMiddlewareForStarlite(
             return {
                 'extra': 'data',
             }
-        elif isinstance(data, dict):
+
+        if isinstance(data, dict):
             return {
                 **data,
                 **{
                     'extra': 'data',
                 },
             }
-        elif isinstance(data, list):
+
+        if isinstance(data, list):
             return [*data, *['extra']]
-        elif isinstance(data, (float, int)):
+
+        if isinstance(data, (float, int)):
             return data + 1
 
         return data + 'changed'
